<?php
/* vim: set expandtab tabstop=4 shiftwidth=4 softtabstop=4: */
/*
 * +----------------------------------------------------------------------+
 *| PHP version 5                                                        |
 *+----------------------------------------------------------------------+
 *| Copyright (c) 2002-2010 Atsuro.Ida                                   |
 *+----------------------------------------------------------------------+
 *+----------------------------------------------------------------------+
 *| Authors: Atsuro.Ida <iserlown@pop06.odn.ne.jp>                       |
 *+----------------------------------------------------------------------+
 *
*	$id:ti_auth.inc (c)A.Ida 2017/3/10 15:37:31
 *
 *
 * 認証管理
 */

require_once("Auth/Auth.php");
require_once('MDB2.php');
require_once("ti_local.inc");
require_once("ti_db.inc");
require_once("ti_Donation.inc");

class AuthCust extends Auth
{
	function drawLogin( $username )
	{
		$status = $this->getStatus();
		require_once 'ti_Login.ihtml';
	}
}

//
// 認証管理クラス
// 注）LAN前提なのでセッションハイジャックとかの対策は無し
//
class Ti_Auth
{
	// Authパラメタ
	var $params = array(
			"dsn" => "",
			"table" => "auth_user_md5",
			"usernamecol" => "username",
			"passwordcol" => "password",
			"db_fields" => "perms",
			"cryptType" => "md5",
	);
	// Auth::Auth
	var $p_auth = null;
	// 権限レベル
	var $p_perms = null;
	// 日本語名
	var $p_cname = null;
	// 前回ログイン
	var $p_lastLogin = null;
	// 今回ログイン
	var $p_nowLogin = null;
	// エラーメッセージ
	var $p_errmsg = '';
	// ダミーAuth
	var $auth = array();
	
	function Ti_Auth()
	{
		// Donation管理オブジェクト生成
		$don = new Ti_Donation();
		$_SESSION["lic"] = $don;
		$lic = new Ti_lic();
		// Authオブジェクト生成
		$dci = new ti_DbConnInfo();
		$this->params["dsn"] = $dci->mkDSN();
		$this->p_auth = new AuthCust("MDB2", $this->params);
		if ($lic->ynIdleSec())
		{
			$this->p_auth->setIdle($lic->key, false);
		}
		$this->p_auth->start();
		if ($this->p_auth->getAuth())
		{
			// 権限レベル
			$this->p_perms = $this->p_auth->getAuthData("perms");
			// for phplib互換
			$this->auth["uname"] = $this->p_auth->getUsername();
			// ユーザ情報取得
			$this->getMUSER($this->p_auth);
			
			// セッション変数初期化
			if (!isset($_SESSION["logined"]) || $_SESSION["logined"] != true)
			{
				// セッション変数初期値
				$_SESSION["g_maxrow"] = $lic->getMaxRow();
				$_SESSION["g_toppid"] = -1;
				$_SESSION["g_toppname"] = "【全て】";
				// 実施画面モード初期値
				$imode = "VIEW";
				if ($lic->ynRunCard())
				{
					$imode = "CARD";
				}
				$_SESSION["imode"] = $imode;
			}
			$_SESSION["logined"] = true;
		}
		else
		{
			exit(0);
		}
	}
	
	function sessInit($vname, $val)
	{
		if (!isset($_SESSION[$vname]))
		{
			$_SESSION[$vname] = $val;
		}
	}

	// MUSERテーブルの情報取得・更新
	function getMUSER($auth)
	{
		$res =false;
		if (isset($_SESSION["cname"]))
		{
			$this->p_cname = $_SESSION["cname"];
			$this->p_lastLogin = $_SESSION["lastLogin"];
			$this->p_nowLogin = $_SESSION["nowLogin"];
			$res = true;
		}
		else
		{
			$db = new DB_Ti();
			if (!$db->isError())
			{
				// 現在情報取得
				$uid = $auth->getUsername();
				$sql_s = "SELECT * FROM muser WHERE cid = '$uid'";
				$db->query($sql_s);
				if (!$db->isError())
				{
					if ($db->next_record())
					{
						$this->p_cname = $db->f("cname");
						$this->p_lastLogin = $db->f("dlogin");
						// 日付更新
						$sql_u = "UPDATE muser SET dlogin = 'NOW' WHERE cid = '$uid'";
						$db->query($sql_u);
						if (!$db->isError())
						{
							// 更新結果取得
							$db->query($sql_s);
							if (!$db->isError())
							{
								$this->p_nowLogin = $db->f("dlogin");
								$_SESSION["cname"] = $this->p_cname;
								$_SESSION["lastLogin"] = $this->p_lastLogin;
								$_SESSION["nowLogin"] = $this->p_nowLogin;
								$res = true;
							}
						}
					}
					else
					{
						$this->p_errmsg = "ユーザ情報が見つかりませんでした。\n" . $db->getMessage();
					}
				}
				else
				{
					$this->p_errmsg = "権限レベルを取得できませんでした。\n" . $db->getMessage();
				}
			}
			else
			{
				$this->p_errmsg = "ＤＢに接続できませんでした。\n" . $db->getMessage();
			}
		}
		return $res;
	}
	
	function logout()
	{
		$this->p_auth->logout();
	}
}


//
// 権限レベル管理
//
class Ti_Perms
{
	var $auth;
	var $permissions;
	
	function Ti_Perms($auth)
	{
		global $c_permary;
		
		$this->auth = $auth;
		$this->permissions = $c_permary;
	}
	
	// 判定
	function comp($perm, $needs)
	{
		$ret = false;
		$p = $this->permissions[$perm];
		$n = $this->permissions[$needs];
		if (($p & $n) == $n)
		{
			$ret = true;
		}
		return $ret;
	}
	
	// Authの権限で判定
	function have_perm($perm)
	{
		return $this->comp($this->auth->p_perms, $perm);
	}

	// Authの権限で判定して処理
	function check($perm)
	{
		global $c_aary;
		if (!$this->have_perm($perm))
		{
			$must_have = $c_aary[$perm];
			$dose_have = $c_aary[$this->auth->p_perms];
			$name = $this->auth->p_cname;
			require_once("ti_perminvalid.ihtml");
			exit();
		}
	}
}
