<?php
/* vim: set expandtab tabstop=4 shiftwidth=4 softtabstop=4: */
/*
 *+----------------------------------------------------------------------+
 *| PHP version 5                                                        |
 *+----------------------------------------------------------------------+
 *| Copyright (c) 2002-2014 Atsuro.Ida                                   |
 *+----------------------------------------------------------------------+
 *+----------------------------------------------------------------------+
 *| Authors: Atsuro.Ida <iserlown@pop06.odn.ne.jp>                       |
 *+----------------------------------------------------------------------+
 *
*	$id:ti_bug.inc (c)A.Ida 2017/3/10 15:37:31
 *
 *
 * 障害票情報管理
 */

require_once("ti_funcs.inc");

class ti_day
{
	var $y;
	var $m;
	var $d;
	
	function setVal($y, $m, $d, $now)
	{
		if((strlen($y) == 0) || (strlen($y) == 0) || (strlen($y) == 0))
		{
			$this->setNow($now);
		}else{
			$this->y = $y;
			$this->m = $m;
			$this->d = $d;
		}
	}
	
	function setNow($now)
	{
		$this->y = $now->y;
		$this->m = $now->m;
		$this->d = $now->d;
	}
	
	function getFull()
	{
		$str = sprintf("%04d/%02d/%02d", $this->y, $this->m, $this->d);
		return $str;
	}
}

class ti_bug
{
	var $db;
	var $bid;
	var $pid;
	var $user_id;
	
	var $iid;
	var $nr;
	var $title;
	var $id_b;
	var $issue ;
	var $pheno;
	var $proc;
	var $imp_b;
	var $term;
	var $id_p;
	var $note;
	var $id_a;
	var $ans;
	var $cause;
	var $cope;
	var $kind;
	var $src;
	var $id_c;
	var $conf;
	var $rs;
	var $re;
	
	var $prjName;
	var $clsName;
	var $bugid;		// 障害票番号文字列（PID-BID）
	var $mark;		// マーク有無
	var $attach_b;	// 添付資料数（発行）
	var $attach_a;	// 添付資料数（回答）
	var $apend;		// 追加項目数
	var $hold;		// 保留項目数
	var $repos;		// リポジトリパス
	
	var $now;
	
	function ti_bug($pid, $bid, $iid, $nr, $user_id, $db)
	{
		$this->db = $db;
		$this->bid = $bid;
		$this->pid = $pid;
		$this->iid = $iid;
		$this->nr = $nr;
		$this->user_id = $user_id;
		
		$this->now = new ti_now($db);
		
		$this->issue = new ti_day();
		$this->term = new ti_day();
		$this->ans = new ti_day();
		$this->conf = new ti_day();
		
		$this->rs = $this->re = 0;
		
		if ($bid > 0)
		{
		// 既存
			$this->setDB();
			$this->bugid = sprintf("%04d-%04d", $pid, $bid);
			$this->mark = $this->chkMark();
			$this->attach_b = $this->cntAttach("B");
			$this->attach_a = $this->cntAttach("A");
			$this->apend = $this->cntApendItem();
			$this->hold = $this->cntHoldItem();
		}else{
		// 新規
			$this->setIni();
			$this->bugid = "保存時確定";
			$this->mark = false;
			$this->attach_b = 0;
			$this->attach_a = 0;
			$this->apend = 0;
			$this->hold = 0;
		}
		
		$get = new ti_get($db);
		$this->prjName = nul2sp($get->fullPrjName($pid));
		$this->clsName = nul2sp($get->fullClsName($pid, $this->iid));
		
		$this->repos = $this->getRepos();
	}
	
	// マーク状態取得
	function chkMark()
	{
		$mark= false;
		$sSql = sprintf("SELECT cid FROM tbmark WHERE cid = '%s' AND npid = %d AND nbid = %d", $this->user_id, $this->pid, $this->bid);
		$this->db->query($sSql);
		if($this->db->num_rows() > 0){
			$mark = true;
		}
		return $mark;
	}
	
	// 添付資料数取得
	function cntAttach($kind)
	{
		$sSql = "SELECT count(*) AS cnt FROM tattach ";
		$sSql .= sprintf("WHERE npid = %d AND ckind = '%s' AND niid = %d", $this->pid, $kind, $this->bid);
		$this->db->query($sSql);
		$this->db->next_record();
		$cnt = $this->db->f("cnt");
		return $cnt;
	}
	
	function cntItem_sub($tbName)
	{
		$cnt = 0;
		$sSql  = "SELECT count(*) AS cnt FROM $tbName";
		$sSql .= sprintf(" WHERE ckind = 'B' AND npid_o = %d AND nid_o = %d", $this->pid, $this->bid);
		$this->db->query($sSql);
		$this->db->next_record();
		$cnt = $this->db->f("cnt");
		return $cnt;
	}
	
	// 追加項目数取得
	function cntApendItem()
	{
		return $this->cntItem_sub("titemref");
	}
	
	// 保留項目数取得
	function cntHoldItem()
	{
		return $this->cntItem_sub("titemhld");
	}
	
	// リポジトリパス取得
	function getRepos()
	{
		$repos = "";
		$pid = $this->pid;
		do{
			$sSql  = "SELECT P.nupid, R.curl FROM tproj P";
			$sSql .= " LEFT JOIN trepos R USING(npid)";
			$sSql .= sprintf(" WHERE P.npid = %d", $pid);
			$this->db->query($sSql);
			if ($this->db->next_record()){
				if(strlen($this->db->f("curl")) > 0){
					$repos = $this->db->f("curl");
					break;
				}
				// 無ければ親プロジェクトの情報取得
				$pid = $this->db->f("nupid");
			}else{
				break;
			}
		}while($pid > 0);
		return $repos;
	}
	
	function setDB()
	{
		$sSql = sprintf("SELECT * FROM vbug WHERE npid = %d AND nbid = %d", $this->pid, $this->bid);
		$this->db->query($sSql);
		if ($this->db->next_record())
		{
			$this->iid = $this->db->f("niid");
			$this->nr = $this->db->f("nround");
			$this->title = htmlspecialchars($this->db->f("ctitle"));
			$this->id_b = $this->db->f("cid_b");
			$this->issue->setVal($this->db->f("cissue_y"), $this->db->f("cissue_m"), $this->db->f("cissue_d"), $this->now);
			$this->pheno = htmlspecialchars($this->db->f("cpheno"));
			$this->proc = htmlspecialchars($this->db->f("cproc"));
			$this->imp_b = $this->db->f("nimp_b");
			$this->term->setVal($this->db->f("cterm_y"), $this->db->f("cterm_m"), $this->db->f("cterm_d"), $this->now);
			$this->id_p = $this->db->f("cid_p");
			$this->note = htmlspecialchars($this->db->f("cnote"));
			$this->id_a = $this->db->f("cid_a");
			if(is_null($this->id_a)){ $this->id_a = $this->user_id; }
			$this->ans->setVal($this->db->f("cans_y"), $this->db->f("cans_m"), $this->db->f("cans_d"), $this->now);
			$this->cause = htmlspecialchars($this->db->f("ccause"));
			$this->cope = htmlspecialchars($this->db->f("ccope"));
			$this->kind = $this->db->f("nkind");
			$this->src = htmlspecialchars($this->db->f("csrc"));
			$this->id_c = $this->db->f("cid_c");
			$this->conf->setVal($this->db->f("cconf_y"), $this->db->f("cconf_m"), $this->db->f("cconf_d"), $this->now);
		}
		$sSql  = sprintf("SELECT * FROM trev WHERE npid = %d AND nbid = %d AND nadd = 1", $this->pid, $this->bid);
		$this->db->query($sSql);
		if($this->db->next_record()){
			$this->rs = $this->db->f("nrs");
			$this->re = $this->db->f("nre");
		}
	}
	
	function setIni()
	{
		$this->title = "";
		$this->id_b = $this->user_id;
		$this->issue->setNow($this->now);;
		$this->pheno = "";
		$this->proc = "";
		$this->imp_b = 2;
		$this->term->setNow($this->now);
		$this->id_p = "";
		$this->note = "";
		$this->id_a = $this->user_id;
		$this->ans->setNow($this->now);
		$this->cause = "";
		$this->cope = "";
		$this->kind = 4;	// 「プログラムミス」
		$this->src = "";
		$this->id_c = "";
		$this->conf->setNow($this->now);
	}
}
?>
