<?php
/* vim: set expandtab tabstop=4 shiftwidth=4 softtabstop=4: */
/*
 * +----------------------------------------------------------------------+
 *| PHP version 5                                                        |
 *+----------------------------------------------------------------------+
 *| Copyright (c) 2002-2010 Atsuro.Ida                                   |
 *+----------------------------------------------------------------------+
 *+----------------------------------------------------------------------+
 *| Authors: Atsuro.Ida <iserlown@pop06.odn.ne.jp>                       |
 *+----------------------------------------------------------------------+
 *
*	$id:ti_bugref.inc (c)A.Ida 2017/3/10 15:37:31
 *
 * 項目と障害票の関連つけ関連共通クラス
 * 「追加項目」と「ホールド」で使用
 */
class ti_bugref
{
	var $mode;
	var $db;
	var $pid;
	var $iids;
	var $tbName;
	var $errstr;
    var $nr;
	
	//
	// mode : 1->追加項目 2->ホールド
	// pid  : 対象項目のPID
	// iids : 項目ID群のカンマ区切り文字列
	// plspids : 検索拡大対象PIDのユーザ入力文字列
	//
	function ti_bugref($db, $pid, $iid)
	{
		$this->db = $db;
		$this->pid = $pid;
		$this->iid = $iid;
		$sSql = sprintf("SELECT COALESCE(nround,0) AS nr FROM tproj WHERE npid = %d", $pid);
		$db->query($sSql);
		$db->next_record();
		$this->nr = $db->f("nr") + 1;
	}
	
	// 障害票一覧のHTML-TABLEを作成
	function mkList()
	{
		$html = <<< HEADER
		<table class='view' cellspacing=0>
		<tr>
		<th nowrap><font size=-1>項番</font></th>
		<th nowrap><font size=-1>選</font></th>
		<th nowrap><font size=-1>障害票番号</font></th>
		<th nowrap><font size=-1>表題</font></th>
		<th nowrap><font size=-1>状態</font></th>
		</tr>
HEADER;
		
		$sSql  = "SELECT B.npid, B.nbid, B.ctitle, toBugStat(B.cid_a, B.cid_c) AS bugstat, R.niid, R.nround ";
		$sSql .= "FROM vbug2 B LEFT JOIN tbugref R ON B.npid = R.npid AND B.nbid = R.nbid AND R.niid = " . $this->iid . " ";
		$sSql .= sprintf("WHERE B.npid = %d AND B.niid <> %d AND (R.niid is NULL OR R.niid = %d) ", $this->pid, $this->iid, $this->iid);
		$sSql .= "ORDER BY B.nbid";
		$this->db->query($sSql);
		$row = 1;
		while($this->db->next_record()){
			$even = "odd";
			if($row%2 == 0){
				$even = "even";
			}
			$html .= sprintf("<tr class=%s id=r%d onMouseOver='ovrColor(%d);' onMouseOut='outColor(%d);'>\n", $even, $row, $row, $row);
			// 項番
			$html .= sprintf("<td class=\"bd caption\" align=\"right\"><font size=\"-1\">%03d</font></td>\n", $row);
			// 選
			$sck = ($this->db->f("niid") != null) ? " CHECKED" : "";
			$sck .= (($sck != "") && ($this->nr != $this->db->f("nround"))) ? " onclick='nooff();this.checked=true;' style='background-color: gray'" : "";
			$html .= sprintf("<td class=\"bd view\"><input type=\"checkbox\" name=\"ckb[]\" value=\"%d\"%s></td>\n", $this->db->f("nbid"), $sck);
			// 番号
			$html .= sprintf("<td class=\"bd view\">%04d-%04d</td>\n", $this->db->f("npid"), $this->db->f("nbid"));
			// 表題
			$html .= sprintf("<td class=\"bd view\">%s</td>\n", htmlspecialchars($this->db->f("ctitle"),0));
			// 状態
			$html .= sprintf("<td class=\"bd view\">%s</td>\n", $this->db->f("bugstat"));

			$html .= "</tr>\n";
			$row++;
		}
		$html .= "</table>\n";
	
		return $html;
	}

	function UpdateDB($bids)
	{
		$this->db->query("BEGIN");
		// 設定済み情報を削除
		$sSql  = "DELETE FROM tbugref WHERE ";
		$sSql .= sprintf("npid = %d AND niid = %d AND nround = %d", $this->pid, $this->iid, $this->nr);
		$res = $this->db->query($sSql);
		if($res){
			if(count($bids) > 0){
				foreach($bids as $bid)
				{
					// 既存確認
					$sSql = sprintf("SELECT count(*) as cnt FROM tbugref WHERE npid = %d AND nbid = %d AND niid = %d", $this->pid, $bid, $this->iid);
					$this->db->query($sSql);
					$this->db->next_record();
					if($this->db->f("cnt") == 0){
					// 新規登録
						$sSql = "INSERT INTO tbugref (npid, nbid, niid, nround) ";
						$sSql .= sprintf("VALUES (%d, %d, %d, %d);", $this->pid, $bid, $this->iid, $this->nr);
						$res = $this->db->query($sSql);
						if($res == FALSE){
							$this->errstr = "データの変更に失敗しました(2)。";
							break;
						}
					}
				}
			}
		}else{
			$this->errstr = "データの変更に失敗しました(1)。";
		}
		if($res){
			$this->db->query("COMMIT");
		}else{
			$this->db->query("ROLLBACK");
		}
		
		return $res;
	 }
}
?>

