<?php
/* vim: set expandtab tabstop=4 shiftwidth=4 softtabstop=4: */
/*
 * +----------------------------------------------------------------------+
 *| PHP version 5                                                        |
 *+----------------------------------------------------------------------+
 *| Copyright (c) 2002-2010 Atsuro.Ida                                   |
 *+----------------------------------------------------------------------+
 *+----------------------------------------------------------------------+
 *| Authors: Atsuro.Ida <iserlown@pop06.odn.ne.jp>                       |
 *+----------------------------------------------------------------------+
 *
*	$id:ti_db.inc (c)A.Ida 2017/3/10 15:37:31
 *
 *
 * データベース制御
 */
require_once("MDB2.php");
require_once("ti_local.inc");

//
// PEAR::MDB2をPHPLibのように使うためのクラス
//
class DB_Ti
{
	var $p_db = null;
	var $p_errmsg = "";
	var $p_err = false;
	var $p_rs;
	var $p_row;
	var $p_num;
	var $p_sql;
	var $errObj;

	var $Halt_On_Error = "off";
	var $Errno = 0;
	var $Error = "";
	var $ErDetail = "";
	
	function DB_Ti($buff=false)
	{
		$cinfo = new Ti_DbConnInfo();
		$opts = array(
			'debug' => 2,
			'result_buffering' => $buff,
		);
		// 接続
		$db = MDB2::factory($cinfo->dsn_ary, $opts);
		if (!MDB2::isError($db)) {
			$db->setFetchMode(MDB2_FETCHMODE_ASSOC);	// カラム名でデータ参照する
			$this->p_db = $db;
		}else{
			$this->setErr("ＤＢに接続できませんでした。", $db);
		}
	}
	
	function setErr($msg, $o)
	{
		$this->Error = $msg . " \n" . $o->getMessage();
		$this->Errno = $o->getCode();
		$this->p_err = true;
		$this->ErrDetail = $o->getUserInfo() . " \n" . $o->getDebugInfo();
	}
	
	function isError()
	{
		return $this->p_err;
	}
	
	function printErr(){
		print("<p class='dberor'>$this->Error</p>\n");
		print("<p>$this->p_sql</p>\n");	// #DEBUG
	}
	
	// 重複エラー判定
	function isDupeError()
	{
		$ret = false;
		if ($this->isError())
		{
			if ($this->Errno == MDB2_ERROR_CONSTRAINT)
			{
				$ret = true;
			}
		}
		return $ret;
	}
	
	function getMessage()
	{
		return $this->Error;
	}
	
	function judgeQueryType($sql)
	{
		$res = 0;
		$ary = explode(" ", $sql, 2);
		$cmd = strtoupper($ary[0]);
		switch($cmd){
		case "INSERT":
		case "UPDATE":
		case "DELETE":
			$res = 1;
			break;
		default:
			$res = 0;
			break;
		}
		return $res;
	}
	
	// 問い合わせ
	function query($sql)
	{
		$ret = true;
		$this->p_err = false;
		switch($this->judgeQueryType($sql))
		{
		case 1:		// UPDATE INSERT DELETE
			$res = $this->p_db->exec($sql);
			$this->p_num = $res;
			break;
		default:	// SELECT
			$res = $this->p_db->query($sql);
			$this->p_rs = $res;
			break;
		}
		if (MDB2::isError($res))
		{
			$this->setErr("ＤＢアクセスに失敗しました。", $res);
			$this->p_sql = $sql;
			$this->errObj = $res;
			$ret = false;
		}
		return $ret;
	}
	
	// フェッチ
	function next_record()
	{
		if (MDB2::isError($this->p_rs))
		{
			$this->p_row = false;
		}else{
			$this->p_err = false;
			$this->p_row = $this->p_rs->fetchRow();
		}
		return $this->p_row;
	}
	
	// データ参照
	function f($colName)
	{
		$res = $this->p_row[$colName];
		if (MDB2::isError($res))
		{
			$this->setErr("カラム($colName)の参照に失敗しました。", $res);
		}
		return $res;
	}
	
	// 現在日時取得
	function getNow()
	{
		$res = null;
		$sql = "SELECT TIMESTAMP 'NOW' AS dnow";
		$this->query($sql);
		if (!$this->isError())
		{
			$res = $this->f("dnow");
		}
		return $res;
	}
	
	// 行移動
	// result_bufferingがtrueでなければならない
	function seek($rowNo)
	{
		$this->p_err = false;
		$res = $this->p_rs->seek($rowNo);
		if (MDB2::isError($res))
		{
			$this->setErr("SEEK($rowNo)に失敗しました。", $res);
		}
		return $res;
	}
	
	// 問い合わせ結果行数
	// result_bufferingがtrueでなければならない
	function num_rows()
	{
		return $this->p_rs->numRows();
	}
	
	// 影響した行数
	// result_bufferingがtrueでなければならない
	function affected_rows()
	{
		return $this->p_num;
	}
}

//
// ＤＢ接続情報管理
//
class Ti_DbConnInfo
{
	var $dsn_ary;
	
	function Ti_DbConnInfo()
	{
		global $c_dsnary;
		$this->dsn_ary = $c_dsnary;
	}
	
	function IsValid($s)
	{
		$ret = false;
		if (isset($this->dsn_ary[$s]))
		{
			if (strlen($this->dsn_ary[$s]) > 0)
			{
				$ret = true;
			}
		}
		return $ret;
	}
	
	function mkDSN()
	{
		$dsn = $this->dsn_ary['phptype'] . "://";
		if ($this->IsValid('username'))
		 {
			$dsn .= $this->dsn_ary['username'];
			if($this->IsValid('password'))
			{
				$dsn .= ":" . $this->dsn_ary['password'];
			}
			if($this->IsValid('hostspec'))
			{
				$dsn .= "@";
			}
		}
		if ($this->IsValid('hostspec'))
		{
			$dsn .= $this->dsn_ary['hostspec'];
		}
		if ($this->IsValid('database'))
		{
			$dsn .= "/" . $this->dsn_ary['database'];
		}
		return $dsn;
	}
}

?>
