<?php
/*
*	$id:ti_graph.inc (c)A.Ida 2017/3/10 15:37:31
*	グラフ計算クラス
*/
class ti_graph {
	// キャンパスサイズ
	var $p_can_x;
	var $p_can_y;
	// フォントＩＤ
	var $p_fontid;
	// マージン
	var $p_mgn_top;
	var $p_mgn_bottom;
	var $p_mgn_left;
	var $p_mgn_right;
	// 最大値
	var $p_max_x;
	var $p_max_y;
	// 有効サイズ
	var $p_len_x;
	var $p_len_y;
	// ゼロアドレス
	var $p_zero_x;
	var $p_zero_y;
	// バージョン判定
	var $p_phpver;

	// コンストラクタ
	function ti_graph($x, $y){
		$this->p_can_x = $x;
		$this->p_can_y = $y;
		
		// デフォルト値
		$this->p_mgn_top = 10;
		$this->p_mgn_bottom = 75;
		$this->p_mgn_left = 68;
		$this->p_mgn_right = 10;
		$this->p_max_x = 0;
		$this->p_max_y = 0;
		$this->p_fontid = 2;

		// PHPバージョン
		$this->p_phpver = substr(PHP_VERSION,0,1);
	}
	
	// 初期化
	function init($max_x, $max_y, $fontid){
		$this->p_fontid = $fontid;
		$this->p_mgn_left = imagefontwidth($fontid) * 10 + 5;
		$this->p_mgn_bottom = imagefontwidth($fontid) * 10 + 10;
		$this->p_zero_x = $this->p_mgn_left;
		$this->p_zero_y = $this->p_can_y - $this->p_mgn_bottom;
		$this->p_len_x = $this->p_can_x - $this->p_mgn_left - $this->p_mgn_right;
		$this->p_len_y = $this->p_can_y - $this->p_mgn_top - $this->p_mgn_bottom;
		$this->p_max_x = $max_x;
		$this->p_max_y = $max_y;
	}
	
	// 要素番号をＸ座標に変換
	function toX($x_no){
		$x = $this->p_zero_x;
		if($this->p_max_x > 0){
			$x += $this->p_len_x * $x_no / $this->p_max_x;
		}
		return $x;
	}
	
	// 数値をＹ座標に変換
	function toY($y_num){
		$y = $this->p_zero_y;
		if($this->p_max_y > 0){
			$y -= $this->p_len_y * $y_num / $this->p_max_y;
		}
		return $y;
	}

	// 線の幅設定
	function setlinesize($img, $size){
		if($this->p_phpver > 3){
			imagesetthickness($img, $size);
		}
	}

	// 点描画
	function drawPoint($img, $x, $y, $c){
		if($this->p_phpver > 3){
			imagefilledellipse($img, $x, $y, 5, 5, $c);
		}else{
			imagefilledrectangle($img, $x-2, $y-2, $x+2, $y+2, $c);
		}
	}
}
?>
