<?php
/* vim: set expandtab tabstop=4 shiftwidth=4 softtabstop=4: */
/*
 * +----------------------------------------------------------------------+
 *| PHP version 5                                                        |
 *+----------------------------------------------------------------------+
 *| Copyright (c) 2002-2010 Atsuro.Ida                                   |
 *+----------------------------------------------------------------------+
 *+----------------------------------------------------------------------+
 *| Authors: Atsuro.Ida <iserlown@pop06.odn.ne.jp>                       |
 *+----------------------------------------------------------------------+
 *
*	$id:ti_itemref.inc (c)A.Ida 2017/3/10 15:37:31
 *
 * 項目と障害票の関連つけ関連共通クラス
 * 「追加項目」と「ホールド」で使用
 */
class ti_itemref
{
	var $mode;
	var $db;
	var $pid;
	var $iids;
	var $tbName;
	var $errstr;
	
	//
	// mode : 1->追加項目 2->ホールド
	// pid  : 対象項目のPID
	// iids : 項目ID群のカンマ区切り文字列
	// plspids : 検索拡大対象PIDのユーザ入力文字列
	//
	function ti_itemref($db, $mode, $pid, $iids)
	{
		$this->db = $db;
		$this->mode = $mode;
		$this->pid = $pid;
		$this->iids = $iids;
		switch($mode)
		{
		case 1:
			$this->tbName = "titemref";
			break;
		case 2:
			$this->tbName = "titemhld";
			break;
		}
	}
	
	// 追加プロジェクトリストの整理
	function ckPids($pid, $pidpls)
	{
		$sret = "";
		$ary = preg_split("[,:; ]", $pidpls);
		$ary2 = array($pid);
		foreach($ary as $val)
		{
			$val = intval($val);
			if ((is_int($val)) && ($val > 0)){
				array_push($ary2, $val);
			}
		}
		$ary2 = array_unique($ary2);
		if (count($ary2) > 0){
			$sret = implode(", ", $ary2);
		}
		return $sret;
	}
	
	// 障害票一覧のHTML-TABLEを作成
	function mkList($pidpls)
	{
		$pids = $this->ckPids($this->pid, $pidpls);
	
		$html = <<< HEADER
		<table class='view' cellspacing=0>
		<tr>
		<th nowrap><font size=-1>項番</font></th>
		<th nowrap><font size=-1>選</font></th>
		<th nowrap><font size=-1>障害票番号</font></th>
		<th nowrap><font size=-1>表題</font></th>
		<th nowrap><font size=-1>状態</font></th>
		</tr>
HEADER;
		
		// 障害テーブルと既に設定済みの障害票を抽出した問い合わせのUNIONに対してGROUP化問い合わせ
		// NCK列は設定済みなら1
		$sSql  = "SELECT npid, nbid, ctitle, bugstat, sum(FLG) AS NCK FROM (";
		$sSql .= "SELECT B1.npid, B1.nbid, B1.ctitle, toBugStat(B1.cid_a, B1.cid_c) AS bugstat, 0 AS FLG FROM vbug2 B1";
		$sSql .= sprintf(" WHERE npid IN (%s)", $pids);
		$sSql .= " UNION";
		$sSql .= " SELECT DISTINCT B2.npid, B2.nbid, B2.ctitle, toBugStat(B2.cid_a, B2.cid_c) AS bugstat, 1 AS FLG FROM vbug2 B2";
		$sSql .= sprintf(" RIGHT JOIN %s R", $this->tbName);
		$sSql .= " ON  R.ckind = 'B'";
		$sSql .= " AND R.npid_o = B2.npid";
		$sSql .= " AND R.nid_o = B2.nbid";
		$sSql .= sprintf(" WHERE R.npid = %d", $this->pid);
		$sSql .= sprintf(" AND R.niid IN (%s)", $this->iids);
		$sSql .= ") TBLS";
		$sSql .= " GROUP BY npid, nbid, ctitle, bugstat";
		$sSql .= " ORDER BY npid, nbid";
		$this->db->query($sSql);
		$row = 1;
		while($this->db->next_record()){
			$even = "odd";
			if($row%2 == 0){
				$even = "even";
			}
			$html .= sprintf("<tr class=%s id=r%d onMouseOver='ovrColor(%d);' onMouseOut='outColor(%d);'>\n", $even, $row, $row, $row);
			// 項番
			$html .= sprintf("<td class=\"bd caption\" align=\"right\"><font size=\"-1\">%03d</font></td>\n", $row);
			// 選
			$sck = ($this->db->f("nck") > 0) ? " CHECKED" : "";
			$html .= sprintf("<td class=\"bd view\"><input type=\"checkbox\" name=\"ckb[]\" value=\"B-%d-%d\"%s></td>\n", $this->db->f("npid"), $this->db->f("nbid"), $sck);
			// 番号
			$html .= sprintf("<td class=\"bd view\">%04d-%04d</td>\n", $this->db->f("npid"), $this->db->f("nbid"));
			// 表題
			$html .= sprintf("<td class=\"bd view\">%s</td>\n", htmlspecialchars($this->db->f("ctitle"),0));
			// 状態
			$html .= sprintf("<td class=\"bd view\">%s</td>\n", $this->db->f("bugstat"));

			$html .= "</tr>\n";
			$row++;
		}
		$html .= "</table>\n";
	
		return $html;
	}
	
	// bids: 障害票番号文字列(KIND-PID-BID)の配列
	function UpdateDB($bids)
	{
		$this->db->query("BEGIN");
		// 設定済み情報を削除（CKIND=Bのみが前提）
		$sSql  = sprintf("DELETE FROM %s WHERE", $this->tbName);
		$sSql .= sprintf(" npid = %d AND niid IN (%s) AND ckind = 'B'", $this->pid, $this->iids);
		$res = $this->db->query($sSql);
		if($res){
			if(count($bids) > 0){
				// 対象項目配列
				$iary = explode(",", $this->iids);
				// 新規設定
				foreach($iary as $iid)
				{
					$nadd = 0;
					foreach($bids as $bid)
					{
						$nadd++;
						// value分解
						$vary = explode("-", $bid);
						// 登録
						$sSql  = sprintf("INSERT INTO %s (npid, niid, ckind, nadd, npid_o, nid_o) VALUES", $this->tbName);
						$sSql .= sprintf(" (%d, %d, '%s', %d, %d, %d)", $this->pid, $iid, $vary[0], $nadd, $vary[1], $vary[2]);
						$res = $this->db->query($sSql);
						if($res == FALSE){
							$this->errstr = "デ−タの変更に失敗しました(2)。";
							break;
						}
					}
					if($res == FALSE){
						break;
					}
				}
			}
		}else{
			$this->errstr = "データの変更に失敗しました(1)。";
		}
		if($res){
			$this->db->query("COMMIT");
		}else{
			$this->db->query("ROLLBACK");
		}
		
		return $res;
	}
}
?>
