--
-- $id:Cr_tifunc.sql (c)A.Ida 2017/3/10 15:37:32
-- テスト項目運用システム V0.5用
-- 再実行しても影響の無いもの
--

--
-- 索引作成
--

-- 同名のプロジェクト作成抑制
DROP INDEX ckProjDup;
CREATE UNIQUE INDEX ckProjDup
ON tproj
(nupid, cpname);

-- tattach本来の主キー
DROP INDEX ckAttachKey;
CREATE UNIQUE INDEX ckAttachKey
ON tattach
(NATTACH);

-- 相談 結合性能対策 重複回答抑制用 V0.3.4 change unique V0.4
DROP INDEX joConvers;
CREATE UNIQUE INDEX joConvers
ON tconvers
(nuckey);

DROP INDEX joConvers2;
CREATE INDEX joConvers2
ON tconvers
(npid);

-- 関連テーブル性能対策用 V0.4
DROP INDEX joItemRef;
CREATE INDEX joItemRef
ON titemref
(ckind, npid_o, nid_o);

--
-- ビューとファンクション
--

-- プロジェクトのフルネーム生成
CREATE or REPLACE FUNCTION get_fullpname(INTEGER, TEXT) RETURNS TEXT AS '
declare
	p1 alias for $1;
	p2 alias for $2;
	sret TEXT;
	prow tproj%ROWTYPE;
begin
	sret := p2;
	if p1 > 0 then
		SELECT * INTO prow FROM tproj WHERE tproj.npid = p1;
		if length(sret) = 0 then
			sret := prow.cpname;
		else
			sret := prow.cpname || '' - '' || sret;
		end if;
		sret := get_fullpname(prow.nupid, sret);
	end if;
	return sret;
end;
' LANGUAGE 'plpgsql';

-- １行抽出 append V0.4.1
CREATE or REPLACE FUNCTION oneline(TEXT) RETURNS TEXT AS 
$BODY$
declare
	strin alias for $1;
	sret TEXT;
	indx INTEGER;
begin
	indx := position(E'\r' in strin);
	if indx = 0 then
		indx := position(E'\n' in strin);
	end if;
	if indx > 0 then
		if indx = 1 then
			sret := ' ';
		else
			sret := substring(strin from 1 for indx-1);
		end if;
	else
		sret := strin;
	end if;
	return sret;
end;
$BODY$
 LANGUAGE 'plpgsql';

-- 分類のフルネーム生成 exchenge V0.4.1
CREATE or REPLACE FUNCTION get_fullcname(INTEGER, INTEGER, TEXT) RETURNS TEXT AS '
declare
	pid alias for $1;
	p1  alias for $2;
	p2  alias for $3;
	sret TEXT;
	prow tclass%ROWTYPE;
begin
	sret := oneline(p2);
	if p1 > 0 then
		SELECT * INTO prow FROM tclass WHERE npid = pid AND tclass.ncid = p1;
		sret := oneline(prow.cclass) || '' - '' || sret;
		sret := get_fullcname(pid, prow.nucid, sret);
	end if;
	return sret;
end;
' LANGUAGE 'plpgsql';

-- 項目に関連した障害の抽出
CREATE or REPLACE FUNCTION get_Bugs(INTEGER, INTEGER, INTEGER) RETURNS TEXT AS '
declare
	pid alias for $1;	-- プロジェクト番号
	iid alias for $2;	-- 項目番号
	nr alias for $3;	-- 実施回数
	sret TEXT;
	prow tbug%ROWTYPE;
begin
	sret := '''';
	for prow IN SELECT * FROM tbug WHERE npid = pid AND niid = iid AND nround = nr loop
		if sret != '''' then
			sret := sret || '':'';
		end if;
		sret := sret || to_char(prow.npid, ''FM0999'') || ''-'' || to_char(prow.nbid, ''FM0999'');
	end loop;
	return sret;
end;
' LANGUAGE 'plpgsql';

-- 配下のプロジェクトを全てカンマ区切りで抽出
CREATE or REPLACE FUNCTION get_Children(INTEGER, TEXT) RETURNS TEXT AS '
declare
	upid alias for $1;
	tout alias for $2;
	sret TEXT;
	pid INTEGER;
	prow tproj%ROWTYPE;
begin
	sret := tout;
	if upid > 0 then
		for prow IN SELECT * FROM tproj WHERE nupid = upid loop
			if length(sret) > 0 then
				sret := sret || '', '';
			end if;
			sret := sret || prow.npid;
			sret := get_Children(prow.npid, sret);
		end loop;
	end if;
	return sret;
end;
' LANGUAGE 'plpgsql';

-- 配下の分類を全てカンマ区切りで抽出
CREATE or REPLACE FUNCTION get_ClassChildren(INTEGER, INTEGER, TEXT) RETURNS TEXT AS '
declare
	pid  alias for $1;
	ucid alias for $2;
	tout alias for $3;
	sret TEXT;
	prow tclass%ROWTYPE;
begin
	sret := tout;
	if ucid > 0 then
		for prow IN SELECT * FROM tclass WHERE npid = pid AND nucid = ucid loop
			if length(sret) > 0 then
				sret := sret || '', '';
			end if;
			sret := sret || prow.ncid;
			sret := get_ClassChildren(pid, prow.ncid, sret);
		end loop;
	end if;
	return sret;
end;
' LANGUAGE 'plpgsql';

-- 下位プロジェクトの存在チェック
CREATE or REPLACE FUNCTION chkLProj(INTEGER) RETURNS TEXT AS '
declare
	pid alias for $1;
	prow tproj%ROWTYPE;
begin
	SELECT * INTO prow FROM tproj T
	WHERE nupid = pid;
	if FOUND then
		return ''1'';   -- 下位プロジェクト有り
	else
		return ''0'';   -- 下位プロジェクト無し
	end if;
end;
' LANGUAGE 'plpgsql';

-- 下位プロジェクトに属すメンバの確認
CREATE or REPLACE FUNCTION chkUsedId(INTEGER, TEXT) RETURNS TEXT AS '
declare
	pid alias for $1;
	id alias for $2;
	sret TEXT;
	prow tpmember%ROWTYPE;
begin
	if char_length(id) > 0 then
		SELECT * INTO prow 
		FROM tpmember 
		WHERE 
			npid in (SELECT npid FROM tproj WHERE nupid = pid) AND
			cid = id;
		if FOUND then
			return ''1''; -- 使用済
		else
			return ''0''; -- 未使用
		end if;
	else
		return ''0'';	 -- 対象外
	end if;
end;
' LANGUAGE 'plpgsql';

-- ユーザの登録状況確認
CREATE or REPLACE FUNCTION chkUser(VARCHAR) RETURNS TEXT AS '
declare
	id alias for $1;
	prow tpmember%ROWTYPE;
begin
	SELECT * INTO prow FROM tpmember T
	WHERE cid = id;
	if FOUND then
		return ''1'';   -- 使用中
	else
		return ''0'';   -- 未使用
	end if;
end;
' LANGUAGE 'plpgsql';

-- 分類の階層数をカウント
CREATE or REPLACE FUNCTION cntClass(INTEGER, INTEGER, INTEGER) RETURNS INTEGER AS '
declare
	pid alias for $1;
	cid alias for $2;
	cnt alias for $3;
	rcnt INTEGER;
	prow tclass%ROWTYPE;
begin
	rcnt := cnt;
	if cid > 0 then
		SELECT * INTO prow FROM tclass WHERE npid = pid AND tclass.ncid = cid;
		rcnt := rcnt + 1;
		rcnt := cntClass(pid, prow.nucid, rcnt);
	end if;
	return rcnt;
end;
' LANGUAGE 'plpgsql';

-- NULLを考慮した日付文字変換
CREATE or REPLACE FUNCTION to_dchar(DATE) RETURNS TEXT AS '
declare
	din alias for $1;
	ret text;
begin
	ret := coalesce(to_char(din, ''YYYY/MM/DD''), ''&nbsp''); 
	return ret;
end;
' LANGUAGE 'plpgsql';

-- ""を考慮した文字日付変換
CREATE or REPLACE FUNCTION to_ndate(TEXT) RETURNS DATE AS '
declare
	din alias for $1;
	ret date;
begin
	if din = '''' then
		ret := null;
	else
		ret := to_date(din, ''YYYY/MM/DD'');
	end if;
	return ret;
end;
' LANGUAGE 'plpgsql';

-- NULLなら&nbspに変換
CREATE or REPLACE FUNCTION nsp(TEXT) RETURNS TEXT AS '
declare
	tin alias for $1;
	ret text;
begin
	ret := coalesce(tin, ''&nbsp''); 
	return ret;
end;
' LANGUAGE 'plpgsql';

-- SYSDATE() 
CREATE or REPLACE FUNCTION sysdate() RETURNS DATE AS '
	SELECT to_date(to_char(CURRENT_TIMESTAMP,''YYYY/MM/DD HH24:MI:SS''), ''YYYY/MM/DD HH24:MI:SS'')
' LANGUAGE 'sql';

-- ID名前変換
CREATE or REPLACE FUNCTION toUName(VARCHAR) RETURNS VARCHAR AS '
	SELECT cname FROM muser WHERE cid = $1
' LANGUAGE 'sql';

-- プロジェクト名変換
CREATE or REPLACE FUNCTION toPName(INTEGER) RETURNS TEXT AS '
	SELECT get_fullpname($1, '''')
	FROM tproj
	WHERE npid = $1
' LANGUAGE 'sql';

-- 項目数
CREATE or REPLACE FUNCTION toINum(INTEGER) RETURNS INT8 AS '
	SELECT count(*)  
	FROM titem
	WHERE npid = $1
' LANGUAGE 'sql';

-- 障害ステータス
CREATE or REPLACE FUNCTION toBugStat(VARCHAR(32), VARCHAR(32)) RETURNS TEXT AS '
declare
	cid_a alias for $1;
	cid_c alias for $2;
	ret text;
begin
	ret := ''済'';
	if cid_c is null then
		ret := ''未確認'';
	end if;
	if cid_a is null then
		ret := ''未回答'';
	end if;
	return ret;
end;
' LANGUAGE 'plpgsql';


-- 指定階層のプロジェクトＩＤを通知
-- RETURN TEXT -> INTEGER at V0.3
DROP VIEW vproj;
CREATE or REPLACE FUNCTION get_nproj(INTEGER, INTEGER, INTEGER, INTEGER) RETURNS INTEGER AS '
declare
	pid  alias for $1;
	upid alias for $2;
	slvl alias for $3;
	dlvl alias for $4;
	prow tproj%ROWTYPE;
	upid_w INTEGER;
	nret INTEGER;
begin
	upid_w := upid;
	if slvl < dlvl then
		return 0;
	end if;
	if slvl = dlvl then
		nret := pid;
	else
		loop	
			SELECT * INTO prow FROM tproj WHERE npid = upid_w;
			if prow.nlvl <= dlvl then
				nret := prow.npid;
				exit;
			else
				upid_w := prow.nupid;
			end if;
		end loop;
	end if;
	return nret;
end;
' LANGUAGE 'plpgsql';

-- 分類の階層を横配置させたtclass
-- DROP VIEW vproj;
CREATE VIEW vproj AS
SELECT 
	get_nproj(npid, nupid, nlvl, 1) AS n1,
	get_nproj(npid, nupid, nlvl, 2) AS n2,
	get_nproj(npid, nupid, nlvl, 3) AS n3,
	get_nproj(npid, nupid, nlvl, 4) AS n4,
	get_nproj(npid, nupid, nlvl, 5) AS n5,
	get_nproj(npid, nupid, nlvl, 6) AS n6,
	get_nproj(npid, nupid, nlvl, 7) AS n7,
	get_nproj(npid, nupid, nlvl, 8) AS n8,
	get_nproj(npid, nupid, nlvl, 9) AS n9,
	*
FROM
	tproj
;

-- tpmemberにユーザ名を追加
DROP VIEW vpmember;
CREATE VIEW vpmember AS
SELECT T.npid, T.cid, M.cname
FROM tpmember T, muser M
WHERE T.cid = M.cid;

-- tresultから最終実施以外を省いたもの
DROP VIEW vitem;
DROP VIEW vitem2;
DROP VIEW vresult;
CREATE VIEW vresult AS
SELECT * FROM tresult R
WHERE
	R.nround = (
		SELECT MAX(R2.nround) 
		FROM tresult R2
		WHERE 
			R2.npid = R.npid AND
			R2.niid = R.niid
		)
;

-- 指定階層の分類文字列を通知
CREATE or REPLACE FUNCTION get_cclass(INTEGER, INTEGER, INTEGER, INTEGER, TEXT, INTEGER) RETURNS TEXT AS '
declare
	pid  alias for $1;
	cid  alias for $2;
	ucid alias for $3;
	slvl alias for $4;
	cls  alias for $5;
	dlvl alias for $6;
	prow tclass%ROWTYPE;
	ucid_w INTEGER;
	sret TEXT;
begin
	ucid_w := ucid;
	if slvl < dlvl then
		return null;
	end if;
	if slvl = dlvl then
		sret := cls;
	else
		loop	
			SELECT * INTO prow FROM tclass WHERE npid = pid AND ncid = ucid_w;
			if prow.nlvl <= dlvl then
				sret := prow.cclass;
				exit;
			else
				ucid_w := prow.nucid;
			end if;
		end loop;
	end if;
	return sret;
end;
' LANGUAGE 'plpgsql';

-- 指定階層の分類ＩＤを通知
-- v0.2で復帰値をINTEGERに変更（ソートのため）
CREATE or REPLACE FUNCTION get_nclass(INTEGER, INTEGER, INTEGER, INTEGER, INTEGER) RETURNS INTEGER AS '
declare
	pid  alias for $1;
	cid  alias for $2;
	ucid alias for $3;
	slvl alias for $4;
	dlvl alias for $5;
	prow tclass%ROWTYPE;
	ucid_w INTEGER;
	nret INTEGER;
begin
	ucid_w := ucid;
	if slvl < dlvl then
		return 0;
	end if;
	if slvl = dlvl then
		nret := cid;
	else
		loop	
			SELECT * INTO prow FROM tclass WHERE npid = pid AND ncid = ucid_w;
			if prow.nlvl <= dlvl then
				nret := prow.ncid;
				exit;
			else
				ucid_w := prow.nucid;
			end if;
		end loop;
	end if;
	return nret;
end;
' LANGUAGE 'plpgsql';

-- 指定階層の表示順を取得 add v0.2
CREATE or REPLACE FUNCTION get_norder(INTEGER, INTEGER, INTEGER, INTEGER, INTEGER, INTEGER) RETURNS INTEGER AS '
declare
	pid  alias for $1;
	cid  alias for $2;
	ucid alias for $3;
	slvl alias for $4;
	dlvl alias for $5;
	ordr alias for $6;
	prow tclass%ROWTYPE;
	ucid_w INTEGER;
	nret INTEGER;
begin
	ucid_w := ucid;
	if slvl < dlvl then
		return 0;
	end if;
	if slvl = dlvl then
		nret := ordr;
	else
		loop	
			SELECT * INTO prow FROM tclass WHERE npid = pid AND ncid = ucid_w;
			if prow.nlvl <= dlvl then
				nret := prow.norder;
				exit;
			else
				ucid_w := prow.nucid;
			end if;
		end loop;
	end if;
	return nret;
end;
' LANGUAGE 'plpgsql';
	

-- 分類の階層を横配置させたtclass
-- add v0.2 o1〜o9（ソート用）
DROP VIEW vanaclass;
DROP VIEW vclass;
CREATE VIEW vclass AS
SELECT 
	get_cclass(npid, ncid, nucid, nlvl, cclass, 1) AS c1,
	get_nclass(npid, ncid, nucid, nlvl, 1) AS n1,
	get_norder(npid, ncid, nucid, nlvl, 1, norder) AS o1,
	get_cclass(npid, ncid, nucid, nlvl, cclass, 2) AS c2,
	get_nclass(npid, ncid, nucid, nlvl, 2) AS n2,
	get_norder(npid, ncid, nucid, nlvl, 2, norder) AS o2,
	get_cclass(npid, ncid, nucid, nlvl, cclass, 3) AS c3,
	get_nclass(npid, ncid, nucid, nlvl, 3) AS n3,
	get_norder(npid, ncid, nucid, nlvl, 3, norder) AS o3,
	get_cclass(npid, ncid, nucid, nlvl, cclass, 4) AS c4,
	get_nclass(npid, ncid, nucid, nlvl, 4) AS n4,
	get_norder(npid, ncid, nucid, nlvl, 4, norder) AS o4,
	get_cclass(npid, ncid, nucid, nlvl, cclass, 5) AS c5,
	get_nclass(npid, ncid, nucid, nlvl, 5) AS n5,
	get_norder(npid, ncid, nucid, nlvl, 5, norder) AS o5,
	get_cclass(npid, ncid, nucid, nlvl, cclass, 6) AS c6,
	get_nclass(npid, ncid, nucid, nlvl, 6) AS n6,
	get_norder(npid, ncid, nucid, nlvl, 6, norder) AS o6,
	get_cclass(npid, ncid, nucid, nlvl, cclass, 7) AS c7,
	get_nclass(npid, ncid, nucid, nlvl, 7) AS n7,
	get_norder(npid, ncid, nucid, nlvl, 7, norder) AS o7,
	get_cclass(npid, ncid, nucid, nlvl, cclass, 8) AS c8,
	get_nclass(npid, ncid, nucid, nlvl, 8) AS n8,
	get_norder(npid, ncid, nucid, nlvl, 8, norder) AS o8,
	get_cclass(npid, ncid, nucid, nlvl, cclass, 9) AS c9,
	get_nclass(npid, ncid, nucid, nlvl, 9) AS n9,
	get_norder(npid, ncid, nucid, nlvl, 9, norder) AS o9,
	*
FROM
	tclass
;

-- titemとvresultを結合したもの
-- UPDATE V0.3
-- add nround2 V0.3.2
CREATE VIEW vitem AS
SELECT
	I.*,
	CASE WHEN COALESCE(R.nround, 0) >= COALESCE(P.nround, 0) THEN COALESCE(R.nround, 0) ELSE COALESCE(P.nround, 0) END AS nround,
	R.cid_r,
	R.dexec,
	COALESCE(R.nround, 0) AS nround2
FROM
	tproj P,
	titem I
	LEFT JOIN vresult R USING(npid, niid)
WHERE
	P.npid = I.npid
;

-- titemとtresultを結合したもの
-- ダウンロード用
CREATE VIEW vitem2 AS
SELECT
	I.*,
	COALESCE(R.nround, 0) AS nround,
	R.cid_r,
	R.dexec
FROM
	titem I
	LEFT JOIN tresult R USING(npid, niid)
;

-- tbugとtans を外部結合したもの
DROP VIEW vbug;
CREATE VIEW vbug AS
SELECT
	*,
	to_char(dissue, 'YYYY/MM/DD') AS cissue,
	to_char(dissue, 'YYYY') AS cissue_y,
	to_char(dissue, 'MM') AS cissue_m,
	to_char(dissue, 'DD') AS cissue_d,
	to_char(dterm, 'YYYY/MM/DD') AS cterm,
	to_char(dterm, 'YYYY') AS cterm_y,
	to_char(dterm, 'MM') AS cterm_m,
	to_char(dterm, 'DD') AS cterm_d,
	to_char(dans, 'YYYY/MM/DD') AS cans,
	to_char(dans, 'YYYY') AS cans_y,
	to_char(dans, 'MM') AS cans_m,
	to_char(dans, 'DD') AS cans_d,
	to_char(dconf, 'YYYY/MM/DD') AS cconf,
	to_char(dconf, 'YYYY') AS cconf_y,
	to_char(dconf, 'MM') AS cconf_m,
	to_char(dconf, 'DD') AS cconf_d
FROM tbug B LEFT JOIN tans A USING(npid,nbid);

-- カウント用簡易版
DROP VIEW vbug2;
CREATE VIEW vbug2 AS
SELECT
	*
FROM tbug B LEFT JOIN tans A USING(npid,nbid);

-- 現在日を年、月、日の文字列で取得
-- UPDATE for 7.3系
DROP VIEW vdays;
CREATE VIEW vdays AS
SELECT
	to_char(NOW(),'YYYY') AS y,
	to_char(NOW(),'MM') AS m,
	to_char(NOW(),'DD') AS d,
	to_char(NOW(),'YYYY/MM/DD') AS cnow;

-- 上位プロジェクト群をカンマ区切りで取得 add V0.3
CREATE or REPLACE FUNCTION get_Parents(INTEGER, TEXT) RETURNS TEXT AS '
declare
	upid alias for $1;
	tout alias for $2;
	sret TEXT;
	pid INTEGER;
begin
	sret := tout;
	if upid > 0 then
		SELECT nupid INTO pid FROM tproj WHERE npid = upid;
		if pid > 0 then
			if length(sret) > 0 then
				sret := sret || '', '';
			end if;
			sret := sret || pid;
			sret := get_Parents(pid, sret);
		end if;
	end if; 
	return sret;
end;
' LANGUAGE 'plpgsql';

-- 相談ビュー add V0.3.4
DROP VIEW vconvers;
CREATE VIEW vconvers AS
SELECT
	C.*,
	C2.nckey AS nans,	-- 回答の発言ＩＤ
	C3.cid AS cuid		-- 相談者
FROM
	tconvers C
	INNER JOIN tproj P ON P.npid = C.npid AND P.csucce IS null
	LEFT JOIN tconvers C2 ON C2.nuckey = C.nckey
	LEFT JOIN tconvers C3 ON C3.nckey = C.nuckey;

-- 分類分布ビュー add V0.4
CREATE VIEW vanaclass AS
SELECT
	C.npid,
	C.n1,
	C.n2,
	C.n3,
	C.n4,
	C.n5,
	C.n6,
	C.n7,
	C.n8,
	C.n9,
	C.o1,
	C.o2,
	C.o3,
	C.o4,
	C.o5,
	C.o6,
	C.o7,
	C.o8,
	C.o9,
	C.ncid,
	C.nlvl,
	C.cclass,
	count(I.ncid) AS cnt
FROM
	titem I FULL JOIN vclass C USING(npid, ncid)
GROUP BY C.npid, C.ncid, C.nlvl, C.cclass, C.o1, C.o2, C.o3, C.o4, C.o5, C.o6, C.o7, C.o8, C.o9, C.n1, C.n2, C.n3, C.n4, C.n5, C.n6, C.n7, C.n8, C.n9;

-- 担当者として登録されているか確認 V0.3.4 update V0.4
CREATE or REPLACE FUNCTION chkUsedMemb(INTEGER, TEXT) RETURNS TEXT AS '
declare
	pid alias for $1;
	id alias for $2;
	sret TEXT;
	rrow tresult%ROWTYPE;
	irow titem%ROWTYPE;
	brow vbug2%ROWTYPE;
	crow tconvers%ROWTYPE;
begin
	if char_length(id) > 0 then
		SELECT * INTO rrow
		FROM tresult
		WHERE npid = pid AND cid_r = id;
		if NOT FOUND then
			SELECT * INTO irow
			FROM titem
			WHERE npid = pid AND (cid = id OR cid2 = id);
			if NOT FOUND then
				SELECT * INTO brow
				FROM vbug2
				WHERE npid = pid AND (cid_b = id OR cid_a = id OR cid_c = id);
				if NOT FOUND then
					SELECT * INTO crow
					FROM tconvers
					WHERE npid = pid AND (cid = id OR cid_a = id);
					if NOT FOUND then
						return ''0''; -- 未使用
					else
						return ''1''; -- 使用済
					end if;
				else
					return ''1''; -- 使用済
				end if;
			else
				return ''1''; -- 使用済
			end if;
		else
			return ''1''; -- 使用済
		end if;
	else
		return ''0''; -- 対象外
	end if;
end;
' LANGUAGE 'plpgsql';

-- 保留有効判定ビュー add V0.5
DROP VIEW vitemhld;
CREATE VIEW vitemhld AS
SELECT DISTINCT
	H.npid,
	H.niid
FROM
	titemhld H
	LEFT JOIN tans A
	ON (H.npid_o = A.npid AND H.nid_o = A.nbid)
WHERE
	A.npid is NULL
;

-- 障害票参照ビュー add V0.5.1
DROP VIEW vbugref;
CREATE VIEW vbugref AS
SELECT
	NPID,
	NBID,
	NIID,
	NROUND,
	CCAUSE
FROM VBUG2
UNION
SELECT
	R.NPID,
	R.NBID,
	R.NIID,
	R.NROUND,
	B.CCAUSE
FROM TBUGREF R LEFT JOIN VBUG2 B USING(NPID, NBID)
;

-- 障害票状況判定 exchange V0.5.1
CREATE or REPLACE FUNCTION ckBug(INTEGER, INTEGER) RETURNS INTEGER AS '
declare
	pid alias for $1;
	iid alias for $2;	
	ret integer;
	bug_cnt integer;
	ans_cnt integer;
begin
	SELECT count(npid), count(ccause) 
	INTO bug_cnt, ans_cnt
	FROM vbugref
	WHERE npid = pid AND niid = iid;
	if bug_cnt = 0 then
		ret := 0;   -- 障害なし
	else
		if bug_cnt = ans_cnt then
			ret := 1;   -- 全て回答済み
		else
			ret := 2;   -- 未回答あり
		end if;
	end if;
	return ret;
end;
' LANGUAGE 'plpgsql';

-- 項目に関連した障害の抽出 add V0.5.1
CREATE or REPLACE FUNCTION get_Bugs2(INTEGER, INTEGER, INTEGER) RETURNS TEXT AS '
declare
	pid alias for $1;	-- プロジェクト番号
	iid alias for $2;	-- 項目番号
	nr alias for $3;	-- 実施回数
	sret TEXT;
	prow tbug%ROWTYPE;
	rrow tbugref%ROWTYPE;
begin
	sret := '''';
	for prow IN SELECT * FROM tbug WHERE npid = pid AND niid = iid AND nround = nr loop
		if sret != '''' then
			sret := sret || '':'';
		end if;
		sret := sret || to_char(prow.npid, ''FM0999'') || ''-'' || to_char(prow.nbid, ''FM0999'');
	end loop;
	for rrow IN SELECT * FROM tbugref WHERE npid = pid AND niid = iid AND nround = nr loop
		if sret != '''' then
			sret := sret || '':'';
		end if;
		sret := sret || to_char(rrow.npid, ''FM0999'') || ''-'' || to_char(rrow.nbid, ''FM0999'');
	end loop;
	return sret;
end;
' LANGUAGE 'plpgsql';

