<?php
/*
 *	テスト項目運用システム データベース作成スクリプト
 *
 *  CLI版PHP.exeを使用してコマンドラインで実行します。
 *  ex) c:\php\php Make_tidb.sql
 *
*	$id:Make_tidb.php (c)A.Ida 2017/3/10 15:37:32
 */
$g_connstr;
$g_psql;
$g_dbname;
$g_user;
$g_passwd;
$g_skip;

// Windows判定
function isWin()
{
	return (strpos(PHP_OS, "WIN") === false) ? false : true;
}

// コマンドラインメッセージ表示
function print_ex($str)
{
	//if (isWin())
	//{
	//	$str = mb_convert_encoding($str, "SJIS-WIN", "auto");
	//}
	print $str;
}

// 確認応答
function ynMsg($msg)
{
	$res = false;
	$retry = true;
	print_ex($msg);
	while($retry)
	{
		print_ex("\n[Y/n]>");
		$txt = trim(fgets(STDIN));
		switch($txt){
		case "y":
		case "Y":
		case "":
			$retry = false;
			$res = true;
			break;
		case "N":
		case "n":
			$retry = false;
			$res = false;
			break;
		}
	}
	return $res;
}

// パラメタ入力
function input_txt($msg, $def="")
{
	$res = $def;
	print_ex($msg);
	print_ex("\n>");
	$txt = trim(fgets(STDIN));
	if (strlen($txt) > 0)
	{
		$res = $txt;
	}
	return $res;
}	

// 接続情報入力・確認
function connect()
{
	global $g_connstr;
	global $g_dbname;
	global $g_user;
	global $g_passwd;
	$res = false;
	
	$msg = "データベース名を入力して下さい。\n改行のみの場合は「pgti」が使用されます。";
	$g_dbname = input_txt($msg, "pgti");
	if (isWin())
	{
		$msg = "データベースのユーザ名（作成するオブジェクトの所有者）を入力して下さい。\n改行のみの場合は「postgres」が使用されます。";
		$g_user = input_txt($msg, "postgres");
		$msg = "ユーザのパスワードを入力して下さい。";
		$g_passwd = input_txt($msg, "");
	}
	$g_connstr = "dbname=" . $g_dbname;
	if (strlen($g_user) > 0)
	{
		$g_connstr .= " user=" . $g_user;
		if (strlen($g_passwd) > 0)
		{
			$g_connstr .= " password=" . $g_passwd;
		}
	}
	$db = pg_connect($g_connstr);
	if ($db === false)
	{
		print_ex("### 接続できませんでした。 ###\n" );
	}else{
		pg_close($db);
		$res = true;
	}
	return $res;
}

// psqlパス入力
function inputPSQL()
{
	global $g_psql;
	$ext = false;
	$def = "";
	$der = "/";
	$psql = "psql";
	if (isWin())
	{
		$der = "\\";
		$psql = "psql.exe";
		$ver = getPgVer();
		$def = "C:\\Program Files\\PostgreSQL\\$ver\\bin";
	}
	$msg = "psqlがインストールされているディレクトリへのパスを入力して下さい。\n";
	$msg .= "パスが通っている場合は改行のみでかまいません。";
	if (strlen($def) > 0)
	{
		$msg .= "\n「1」を入力すると $def が使用されます。";
	}
	while($ext == false)
	{
		$path = input_txt($msg, "");
		if ($path == 1){ $path = $def; }
		if (strlen($path) > 0)
		{
			$path = rtrim($path, $der) . $der . $psql;
			if (file_exists($path))
			{
				$ext = true;
			}else{
				print_ex("### 「" . $path . "」が見つかりません。再入力して下さい。 ###\n");
			}
		}else{
			$path = $psql;
			$ext = true;
		}
	}
	return $path;
}

// SQLファイルコード選択
function selectCode()
{
	$sjis = false;
	$def = "0";
	if (isWin())
	{
		$var = getPgVer();
		if ($var <= "8.3")
		{
			$sjis = true;
			$def = "1";
		}
		$msg  = "SQLファイルの文字コードを選択してください。\n";
		$msg .= "UTF8なら「0」、S-JISなら「1」を入力します。\n";
		$msg .= $sjis ? "S-JIS" : "UTF8";
		$msg .= "でよければ改行のみでもよいです。\n";
		$msg .= "UTF8だとエラーになってしまう版のPostgreSQLがあったのでその対策です。";
		$ok = false;
		while($ok == false)
		{
			$ok = true;
			$key = input_txt($msg, $def);
			switch($key)
			{
			case "0":
				$sjis = false;
				break;
			case "1":
				$sjis = true;
				break;
			default:
				$ok = false;
				break;
			}
		}
	}
	return $sjis;
}

// PostgreSQL Ver.取得
function getPgVer()
{
	global $g_connstr;
	$ver = "8.4";
	$db = pg_connect($g_connstr);
	if($db){
		$rs = RunSQL($db,"SELECT version()");
		if ($rs)
		{
			$row = pg_fetch_row($rs, 0);
			$ary = explode(" ", $row[0]);
			$vary = explode(".", $ary[1]);
			$ver = $vary[0] . "." . $vary[1];
		}
		pg_close($db);
	}
	return $ver;
}

// SELECT
function RunSQL($db, $sql)
{
	if($db)
	{
		$rs = pg_exec($db, $sql);
	}
	return $rs;
}

// SQLファイル確認・実行
function ExecFile($file, $str, $code=true)
{
	global $g_sjis;
	global $g_psql;
	global $g_dbname;
	global $g_user;
	global $g_passwd;
	global $g_skip;
	$ret = true;
	$g_skip = false;
	
	$msg = $str . "を行います。";
	if (ynMsg($msg))
	{
		$psql = $g_psql;
		if (isWin())
		{
			$psql = "\"" . $g_psql . "\"";
			if ($g_sjis && $code)
			{
				$file = "sjis\\" . $file;
			}
		}
		$cmd = $psql;
		if (strlen($g_user) > 0)
		{
			$cmd .= " -U " . $g_user;
		}
		$cmd .= " -f " . $file . " -d " . $g_dbname;
		exec($cmd, $resAry, $res);
		if($res == 0)
		{
			print_ex($str . "のSQL実行に成功しました。\n");
			print_ex("エラーが表示されていないか確認してください。\n");
		}else{
			$ret = false;
			print_ex("### " . $str . "に失敗しました。 ###\n");
			foreach($resAry as $val)
			{
				print($val . "\n");
			}
		}
	}else{
		$g_skip = true;
	}
	return $ret;
}

		
print_ex("【テスト項目運用システムV0.5.1 ＤＢ作成ツール】\n");
print_ex("データベースのオブジェクト（テーブル、ビュー、ファンクションなど）を作成します。\n");
print_ex("事前にcreatedbでエンコードがUTF8な新規データベースを作成しておく必要があります。\n");
if (!isWin())
{
	print_ex("現在のUNIXログインユーザと同じ名前のPostgreSQLのユーザが所有者になります。\n");
}
print_ex("\n");

// 接続情報入力・確認
$res = connect();
// psqlのパス
$g_psql = inputPSQL();
// コード
$g_sjis = selectCode();
// テーブル・シーケンス
if ($res)
{
	$brun = false;
	// テーブル既存確認
	$sSql = "SELECT * FROM muser";
	$db = pg_connect($g_connstr);
	if($db)
	{
		$rs = @RunSQL($db, $sSql);
		if ($rs)
		{
			print_ex("テーブルが既存するようなのでテーブル作成フェーズをスキップします。\n");
		}else{
			$brun = true;
		}
		pg_close($db);
	}
	if ($brun)
	{
		$res = ExecFile("Cr_titb.sql", "テーブルとシーケンスの作成");
		if ($res && $g_skip == false){
			print_ex("indexを作成する旨のNOTICE:が表示される場合がありますが、これは問題ありません。\n");
		}
	}
}

// ビュー・ファンクション
if ($res)
{
	$res = ExecFile("Cr_tifunc.sql", "ビューとファンクションの作成・再作成");
	if ($res && $g_skip == false){
		print_ex("初回実行時は、does not exist の ERROR: が表示されますが、これは問題ありません。\n");
	}
}

// 参照整合
if ($res)
{
	$res = ExecFile("Cr_const.sql", "参照整合性制約の作成");
}

// 権限付与
if ($res)
{
	$res = ExecFile("Gr_ti.sql", "実行ユーザに権限付与", false);
}

print_ex("終了しました。\n");

?>
