--
-- $id:alt034to04.sql (c)A.Ida 2017/3/10 15:37:32
-- テスト項目運用システム　ＤＢバージョンアップ
-- V0.4で追加したテーブルを作成します。
--

-- メアドテーブル
-- DROP TABLE TMAIL;
CREATE TABLE TMAIL (
	CID		VARCHAR(32)			NOT NULL,
	CBUG	CHAR(1)				NOT NULL,	-- 0:不許可 1:許可 2:全件
	CCONV	CHAR(1)				NOT NULL,	-- 0:不許可 1:許可 2:全件
	CRECOG	CHAR(1)	Default '0' NOT NULL,	-- 現版未使用
	CMAIL	VARCHAR(64) 		NOT NULL,
	PRIMARY KEY(CID)
);

-- メール発信ログ
-- DROP TABLE LMAIL;
CREATE TABLE LMAIL (
	NKIND	INT4			NOT NULL,		-- 1:障害発行 2:回答 3:確認 11:相談
	NPID	INT4			NOT NULL,		-- 
	NID		INT4			NOT NULL,		-- 障害:nbid 相談:nckey
	NADD	INT4			NOT NULL,
	DSEND	TIMESTAMP		NOT NULL,
	CTO		TEXT			NULL,			-- 
	PRIMARY KEY(NKIND,NPID,NID,NADD,DSEND)
);

-- リポジトリテーブル
-- DROP TABLE TREPOS;
CREATE TABLE TREPOS (
	NPID	INT4			NOT NULL,
	CURL	VARCHAR(256)	NOT NULL,
	PRIMARY KEY(NPID)
);

-- リビジョンテーブル
-- DROP TABLE TREV;
CREATE TABLE TREV (
	NPID	INT4	NOT NULL,
	NBID	INT4	NOT NULL,
	NADD	INT4	Default 1	NOT NULL, 
	NRS		INT4	NOT NULL,
	NRE		INT4	NOT NULL,
	PRIMARY KEY(NPID,NBID,NADD)
);

-- 項目関連付けテーブル
-- DROP TABLE TITEMREF;
CREATE TABLE TITEMREF (
	NPID	INT4			NOT NULL,	
	NIID	INT4			NOT NULL,	-- 項目ＩＤ
	CKIND	CHAR(1)			NOT NULL,	-- B:障害票 R:根拠
	NADD	INT4			NOT NULL,
	NPID_O	INT4			NOT NULL,
	NID_O	INT4			NOT NULL,	-- 障害番号など
	PRIMARY KEY(NPID, NIID, CKIND, NADD)
);

-- 分類分布ビュー
-- DROP VIEW vanaclass;
CREATE VIEW vanaclass AS
SELECT
	C.npid,
    C.n1,
    C.n2,
    C.n3,
    C.n4,
    C.n5,
    C.n6,
    C.n7,
    C.n8,
    C.n9,
    C.o1,
    C.o2,
    C.o3,
    C.o4,
    C.o5,
    C.o6,
    C.o7,
    C.o8,
    C.o9,
    C.ncid,
	C.nlvl,
    C.cclass,
    count(I.ncid) AS cnt
FROM
    titem I FULL JOIN vclass C USING(npid, ncid)
GROUP BY C.npid, C.ncid, C.nlvl, C.cclass, C.o1, C.o2, C.o3, C.o4, C.o5, C.o6, C.o7, C.o8, C.o9, C.n1, C.n2, C.n3, C.n4, C.n5, C.n6, C.n7, C.n8, C.n9;

-- 整合制約の追加

-- メールとユーザ
ALTER TABLE tmail ADD CONSTRAINT ref_mail_user
FOREIGN KEY (cid) REFERENCES muser(cid)
ON DELETE CASCADE;

-- リポジトリとプロジェクト
ALTER TABLE trepos ADD CONSTRAINT ref_rep_proj
FOREIGN KEY (npid) REFERENCES tproj(npid)
ON DELETE CASCADE;

-- リビジョンと障害回答
ALTER TABLE trev ADD CONSTRAINT ref_rev_ans
FOREIGN KEY (npid, nbid) REFERENCES tans(npid, nbid)
ON DELETE CASCADE;

-- 障害とマーク add V0.4
ALTER TABLE tbmark DROP CONSTRAINT ref_proj_bmark;
ALTER TABLE tbmark ADD CONSTRAINT ref_bmark_bug
FOREIGN KEY (npid, nbid) REFERENCES tbug(npid, nbid)
ON DELETE CASCADE;

-- 関連と項目 add V0.4
ALTER TABLE titemref ADD CONSTRAINT ref_ref_item
FOREIGN KEY (npid, niid) REFERENCES titem(npid, niid)
ON DELETE CASCADE;


-- 以降は古いバージョンのPostgreSQLでは失敗することがあります。
-- 失敗した場合の影響は次の２点です。
-- 「前回ログイン日付に時刻が表示されない」「実施日に時刻が保存されない」

-- 最終ログイン日付に時刻も含めるように変更
ALTER TABLE muser ALTER COLUMN dlogin TYPE timestamp;

-- テスト実施日に時刻も含めるように変更
DROP VIEW vitem;
DROP VIEW vitem2;
DROP VIEW vresult;
ALTER TABLE tresult ALTER COLUMN dexec TYPE timestamp;
CREATE VIEW vresult AS
SELECT * FROM tresult R
WHERE
    R.nround = (
        SELECT MAX(R2.nround) 
        FROM tresult R2
        WHERE 
            R2.npid = R.npid AND
            R2.niid = R.niid
        )
;
CREATE VIEW vitem AS
SELECT
    I.*,
    CASE WHEN COALESCE(R.nround, 0) >= COALESCE(P.nround, 0) THEN COALESCE(R.nround, 0) ELSE COALESCE(P.nround, 0) END AS nround,
    R.cid_r,
    R.dexec,
    COALESCE(R.nround, 0) AS nround2
FROM
    tproj P,
    titem I
    LEFT JOIN vresult R USING(npid, niid)
WHERE
    P.npid = I.npid
;
CREATE VIEW vitem2 AS
SELECT
    I.*,
    COALESCE(R.nround, 0) AS nround,
    R.cid_r,
    R.dexec
FROM
    titem I
    LEFT JOIN tresult R USING(npid, niid)
;



-- 正常終了後、ご使用の環境に合わせて権限付与のSQLを実行してください。
-- Webサーバの実行ユーザやti_local.incに設定した接続ユーザに合わせて事前に編集が必要な場合があります。
-- 詳細はインストール手順書を参照してください。
-- Linux   : Gr_ti.sql     (install_vine41_php5.htm)
-- Windows : Gr_ti_win.sql (install_win_iis6.htm / install_winxp_apache.htm)

